'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

from django.utils.translation import gettext_lazy as _

from serialbox.flavor_packs import FlavorPackApp


class RandomFlavorpackConfig(FlavorPackApp):
    name = "random_flavorpack"
    verbose_name = _("Random FlavorPack")

    # This field tells the PoolSerializerMeta class to add this field
    # to the pool serializer so that randomized regions auto-magically show up
    # in the pool serializer output.
    @property
    def pool_slug_fields(self):
        return {
            'randomizedregion_set':
            'random_flavorpack.api.serializers.randomizedregion_set'
        }

    # Each flavorpack must supply hyperlink fields for it's serializer fields
    @property
    def pool_hyperlink_fields(self):
        return {
            'randomizedregion_set':
            'random_flavorpack.api.serializers.randomizedregion_hyperlink_set'
        }
    # Each flavorpack must supply number generators for it's regions

    @property
    def generators(self):
        return {
            'random_flavorpack.models.RandomizedRegion':
            'random_flavorpack.generators.random.RandomGenerator'
        }

    @property
    def api_urls(self):
        return ['randomized-regions-list']


'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

import collections

import logging
from django.contrib.auth.models import User, Permission, Group
from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase, APIRequestFactory

from random_flavorpack.generators.random import RandomGenerator
from random_flavorpack.management.commands.load_random_flavorpack_auth import \
    Command
from serialbox.utils import get_region_by_machine_name

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('random_unit_tests')


class RandomTests(APITestCase):
    def setUp(self):
        user = User.objects.create_user(username='testuser',
                                        password='unittest',
                                        email='testuser@seriallab.local')
        Command().handle()
        for permission in Permission.objects.all():
            print(permission.name, permission.codename)
        group = Group.objects.get(name='Pool API Access')
        user.groups.add(group)
        user.save()
        self.client.force_authenticate(user=user)
        self.create_pool()
        self.create_region()
        self.user = user
        self.logging_setup()

    def logging_setup(self):
        import logging
        import sys

        root = logging.getLogger()
        root.setLevel(logging.INFO)

        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.INFO)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        root.addHandler(ch)

    def create_pool(self, data=None, assert_status=status.HTTP_201_CREATED):
        '''
        Ensure we can create a new pool instance.
        '''
        data = data or {
            "readable_name": "Random Unit Test Pool",
            "machine_name": "rutpool1",
            "active": "true",
            "request_threshold": 100
        }
        url = reverse('pool-create')
        response = self.client.post(url, data, format='json')
        logger.info(response.content)
        self.assertEqual(response.status_code, assert_status)

    def create_region(self, data=None, assert_status=status.HTTP_201_CREATED):
        '''
        Ensure we can create a test region for the created pool instance
        '''
        data = data or {
            "pool": "rutpool1",
            "readable_name": "Unit Test Random Region",
            "machine_name": "utrr",
            "active": True,
            "min": 100000,
            "max": 10000000,
            "start": 1000001,
            "current": 1000001
        }
        url = reverse('randomized-regions-list')
        response = self.client.post(url, data, format='json')
        logger.info(response.content)
        self.assertEqual(response.status_code, assert_status)
        return response

    def test_create_region_bad_start(self, data=None,
                                     assert_status=status.HTTP_201_CREATED):
        '''
        Ensure we can create a test region for the created pool instance
        '''
        data = data or {
            "pool": "rutpool1",
            "readable_name": "Bad Unit Test Random Region",
            "machine_name": "utrr2",
            "active": True,
            "min": 1,
            "max": 100,
            "start": 1000001,
            "current": 1000001
        }
        url = reverse('randomized-regions-list')
        response = self.client.post(url, data, format='json')
        logger.info(response.content)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        return response

    def test_create_region_bad_min(self, data=None, \
                                   assert_status=status.HTTP_201_CREATED):
        '''
        Ensure we can create a test region for the created pool instance
        '''
        data = data or {
            "pool": "rutpool1",
            "readable_name": "Bad Unit Test Random Region",
            "machine_name": "utrr2",
            "active": True,
            "min": 122222222,
            "max": 10000000,
            "start": 1000001,
            "current": 1000001
        }
        url = reverse('randomized-regions-list')
        response = self.client.post(url, data, format='json')
        logger.info(response.content)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        return response

    def test_allocate_numbers(self):
        '''
        Ensure we can get numbers from the pool
        '''
        url = reverse('allocate-numbers', args=['rutpool1', '10'])
        response = self.client.get(url, format='json')
        logger.info(response.content)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_region_remaining(self):
        region = get_region_by_machine_name('utrr')
        self.assertEqual(region.remaining, (region.max - region.min),
                         'The region remaining size has not been calculated  '
                         'correctly')

    def test_get_region_by_machine_name(self):
        region = get_region_by_machine_name('utrr')
        self.assertTrue(
            region is not None,
            'Could not look up region %s using '
            'the get_region_by_machine_name utility.')
        logger.info('randomized region was retrieved...')

    def test_generate_10_pseudo_random(self):
        state = 23423
        rg = RandomGenerator()
        numbers = \
            rg.get_random_numbers(state, 1, 999999999999,
                                  size=10, shuffle_numbers=False)[0]
        logger.info(numbers)
        self.assertTrue(
            set(numbers) == {23423, 687196089791, 927714708335, 876175001015,
                             850405147355, 837520220525,
                             831077757110, 415538878555, 620087086125,
                             997239621142})

    def test_shuffle(self):
        state = 23423
        rg = RandomGenerator()
        numbers = rg.get_random_numbers(state, 1, 999999999999, size=10,
                                        shuffle_numbers=True)[0]
        compare = lambda x, y: collections.Counter(x) == collections.Counter(y)
        orig = [23423, 687196089791, 927714708335, 876175001015,
                850405147355, 837520220525,
                831077757110, 415538878555, 620087086125,
                997239621142]
        self.assertFalse(orig[1] == numbers[1] and orig[2] == numbers[2] and
                         orig[3] == orig[3] and orig[4] == orig[4] and
                         orig[5] == orig[5] and
                         orig[6] == orig[6] and
                         orig[7] == orig[7] and
                         orig[8] == orig[8] and
                         orig[9] == orig[9],
                         'The list has not been shuffled')

    def test_get_regions(self):
        factory = APIRequestFactory()
        ret = factory.get('/randomized-regions/')
        print(ret)



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Copyright 2018 SerialLab Corp.  All rights reserved.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Copyright 2018 SerialLab Corp.  All rights reserved.
from django.utils.translation import gettext as _
from django.core.management.base import BaseCommand
from serialbox.models import Pool
from random_flavorpack import models


class Command(BaseCommand):
    help = _(
        'Loads some example pools into the database.'
    )

    def handle(self, *args, **options):
        sp1 = Pool.objects.create(
            readable_name=_('Pharmaprod 20mcg Pills'),
            machine_name='00313000007772',
            active=True,
            request_threshold=1000
        )
        models.RandomizedRegion.objects.create(
            readable_name=_('Pharmaprod 20mcg Pills'),
            machine_name='00313000007772',
            start=239380,
            active=True,
            order=1,
            pool=sp1,
            min=1,
            max=999999999999
        )



'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

from serialbox.errors import RegionBoundaryException

def check_randomized_region_boundaries(randomized_region):
    # type: (RandomizedRegion) -> None
    '''
    Checks the pool of a given randomized region to ensure that no
    two regions have overlapping numbers.

    :param randomized_region: The region to check
    :return: None or will raise a RegionsBoundaryException
    '''
    if randomized_region.min >= randomized_region.max:
        raise RegionBoundaryException(
            'The random region has a malformed boundary.  '
            'The min value may not be greater than '
            'or equal to the the max value.')
    if randomized_region.start > randomized_region.max or \
        randomized_region.start < randomized_region.min:
        raise RegionBoundaryException(
            'The random region may not hav a start value less than '
            'it\'s min value or greater than it\'s max.')



'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

import math
import logging
from gettext import gettext as _
from random import shuffle
from serialbox.generators.common import Generator
from random_flavorpack.generators.common import TAPS
from random_flavorpack import random_flavorpack_settings as settings

logger = logging.getLogger(__name__)


class RandomGenerator(Generator):
    '''
    Can be used to return a double-randomized list of numbers if
    shuffle is specified. Otherwise a pseudo-randomized list,
    according to the start value, is generated
    using a standard linear feedback shift register.
    '''

    def generate(self, request, response, region, size, shuffle_numbers=True):
        '''
        Generates the randomized values based on the region supplied.
        '''
        #: :type region: RandomizedRegion
        response.type = 'random'
        numbers, current = self.get_random_numbers(
            region.current or region.start,
            region.min,
            region.max,
            size,
            shuffle_numbers
        )
        region.current = current
        region.remaining = region.remaining - size
        region.save()
        self.set_number_list(response, numbers)

    def get_settings_module(self):
        return settings

    def _enforce_boundaries(self, response, size, region):
        '''
        Makes sure the count does not exceed the amount remaining.
        '''
        remaining_count = region.remaining
        logger.debug('Remaining count for region %s = %s', region,
                     remaining_count)
        if size >= remaining_count:
            logger.debug('Size exceeded remaining count.')
            region.active = False
            response.fulfilled = (size == remaining_count)
            size = remaining_count + 1
            response.size_granted = size
        return size

    def _validate_range(self, start, minimum, maximum):
        if maximum <= minimum:
            raise ValueError(
                _("The maximum can not be less than the minimum."))
        if start < minimum or start > maximum:
            raise ValueError(
                _("The start must be between the minimum and maximum!"))
        rnrange = maximum - minimum
        return rnrange

    def _get_degree(self, maximum, rnrange):
        degree = math.log(rnrange, 2)
        ceiling = math.ceil(degree)
        if degree == ceiling:
            degree += 1
        else:
            degree = ceiling
        if degree > 168:
            raise ValueError(_("Range can not be larger than 63 bits."))
        if degree < 3:
            degree = 3
        return degree

    def _generate_list(self, start, minimum, rnrange, degree):
        yield start
        start -= (minimum - 1)
        init_seed = start
        taps = sum([1 << (tap - 1) for tap in TAPS[degree]])
        while True:
            # use a mask to get the right-most bit
            least_significant_bit = start & 1
            # shift all the bits to the right
            start >>= 1
            if least_significant_bit:
                start ^= taps
            if start == init_seed:
                # the mini- is complete, exit the loop
                raise StopIteration()
            if start <= rnrange:
                yield start + minimum - 1

    def get_random_numbers(self, start, minimum, maximum, size,
                           shuffle_numbers=False):
        rnrange = self._validate_range(start, minimum, maximum)
        # calculate how many bits the register needs to be
        degree = self._get_degree(maximum, rnrange)
        number_gen = self._generate_list(start, minimum, rnrange, degree)
        numbers = [next(number_gen) for x in range(size)]
        current = next(number_gen)
        if shuffle_numbers:
            shuffle(numbers)
        return numbers, current



'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

import sys
from random import randint

from django.db import models
from django.utils.translation import gettext_lazy as _
from django.core.validators import MinValueValidator

from serialbox import models as sb_models
from random_flavorpack.utils import check_randomized_region_boundaries

class RandomizedRegion(sb_models.Region):
    '''
    # RandomizedRegion
    The randomized region defines a region that notes a start value (which
    itself is random) and a minimum and maximim value for any randomized value
    returned from the region.  Multiple randomized regions may be defined per
    pool but their min and max values may not overlap.
    '''
    # TODO Create a field that calculates how many numbers are remaining
    min = models.BigIntegerField(
        verbose_name=_('Minimum'),
        help_text=_('The minimum value in the randomized region.'),
        default=1,
        validators=[MinValueValidator(0)],
        null=False,
        blank=False)
    max = models.BigIntegerField(
        verbose_name=_('Maximum'),
        help_text=_('The maximum value in the randomized region.'),
        default=sys.maxsize,
        validators=[MinValueValidator(0)],
        null=False,
        blank=False)
    start = models.BigIntegerField(
        verbose_name=_('Start Number'),
        validators=[MinValueValidator(0)],
        help_text=_(
            'The start number will fall somewhere in between the '
            'minimum and maximum numbers.  This number will be randomly'
            ' selected by the system to assure the most randomized '
            'range possible'),
        null=True,
        blank=True)
    current = models.BigIntegerField(
        verbose_name=_('Current'),
        help_text=_('The current number represents the next number in the '
                    'generator.  The current number is used by the system as '
                    'a state variable that, along with the start, '
                    'can be used to re-position the random number generator '
                    'where it left off.'),
        null=True,
        blank=True,
        validators=[MinValueValidator(0)]
    )
    remaining = models.BigIntegerField(
        verbose_name=_('Remaining'),
        help_text=_('The number of remaining serial numbers in this randomized '
                    'region.'),
        null=True,
        blank=True,
        validators=[MinValueValidator(0)],
    )

    def __str__(self):
        '''
        Return the human readable name for any interface elements.
        '''
        return self.readable_name

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):
        if not self.id:
            self.start = self.start or randint(self.min, self.max)
            self.current = self.current or self.start
            self.remaining = self.remaining or self.max - self.min
            self._establish_order()

        check_randomized_region_boundaries(self)
        sb_models.Region.save(
            self,
            force_insert,
            force_update,
            using,
            update_fields)

    def clean(self):
        check_randomized_region_boundaries(self)

    def _establish_order(self):
        '''
        Establish order where there is none. :-)
        '''
        regions = RandomizedRegion.objects.filter(pool=self.pool).aggregate(
            models.Max('order')
        )
        val = regions['order__max'] or 0 + 1
        self.order = val + 1

    class Meta(object):
        app_label = 'serialbox'
        verbose_name = _('Randomized Region')
        verbose_name_plural = _('Randomized Regions')



'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
from dotenv import load_dotenv

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

dotenv_path = os.path.join(os.path.dirname(__file__), '../.env')
load_dotenv(dotenv_path)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'l$+#s2al#vz27ai)q4of)#$oni9&lk&)=0%tnh_5donkkzddnm'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['localhost']

# Application definition

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'quartet_capture',
    'serialbox',
    'rest_framework',
    'random_flavorpack',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

ROOT_URLCONF = 'tests.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            #             os.path.join(os.path.dirname(__file__),
            #                          'serialbox/templates')
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]
WSGI_APPLICATION = 'wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases
DATABASES = {
    'default': {
        # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        # Or path to database file if using sqlite3.
        'NAME': os.environ.get('SERIALBOX_DB'),
        'USER': os.environ.get('SERIALBOX_USER'),
        'PASSWORD': os.environ.get('SERIALBOX_PASSWORD'),
        # Empty for localhost through domain sockets or           '127.0.0.1'
        # for localhost through TCP.
        'HOST': os.environ.get('SERIALBOX_DB_HOST', 'localhost'),
        # Set to empty string for default.
        'PORT': os.environ.get('SERIALBOX_DB_PORT', '5432'),
    }
}


# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/
STATIC_ROOT = os.path.join(os.path.dirname(__file__), 'static')
STATIC_URL = '/static/'

LOGGING_CONFIG = None

# import the django rest framework settings
from serialbox.serialbox_settings import REST_FRAMEWORK

try:
    from random_flavorpack.tests.local_settings import *
except ImportError:
    pass


'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

from django.core.exceptions import ValidationError

from serialbox.api.viewsets import FormMixin
from serialbox import viewsets

from random_flavorpack.models import RandomizedRegion
from random_flavorpack.api import serializers
from serialbox.errors import RegionBoundaryException


class RandomizedRegionViewSet(viewsets.SerialBoxModelViewSet, FormMixin):
    '''

    # Randomized Region API

    The randomized region API supports the following HTTP actions:

    * GET - Retrieve a region using the `machine_name`
    * POST - Create a region by posting a serialized random region in *JSON*
    or *XML* format.
    * PUT - Update a region by posting a serialized random region in *JSON*
    or *XML* format.
    * DELETE - Delete a region by using the `machine_name` of the region.

    '''
    queryset = RandomizedRegion.objects.all()
    lookup_field = 'machine_name'

    def get_serializer_class(self):
        '''
        Return a different serializer depending on the client request.
        '''
        ret = serializers.RandomizedRegionSerializer
        try:
            if self.request.query_params.get('related') == 'true':
                ret = serializers.RandomizedRegionHyperlinkedSerializer
        except AttributeError:
            pass
        return ret

    def create(self, request, *args, **kwargs):
        try:
            return viewsets.ModelViewSet.create(self, request, *args, **kwargs)
        except ValidationError as v:
            raise RegionBoundaryException(v.detail)





'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

default_app_config = 'random_flavorpack.apps.RandomFlavorpackConfig'

__version__ = '2.0.0'



'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

from rest_framework import serializers

from serialbox import models as pool_models
from serialbox.api.serializers import RegionSerializer
# from serialbox.api import serializers as sb_serializers

from random_flavorpack import models as fp_models

##
# These fields will be added to the pool serializer automatically since
# they are specified in the pool_slug_fields and the pool_hyperlinked_fields
# values in the random_flavorpack.apps.RandomFlavorpackConfig module.
##
randomizedregion_set = serializers.SlugRelatedField(
    many=True,
    queryset=fp_models.RandomizedRegion.objects.all(),
    slug_field='machine_name',
    required=False
)

randomizedregion_hyperlink_set = serializers.HyperlinkedRelatedField(
    many=True,
    read_only=True,
    view_name='randomized-regions-detail',
    lookup_field='machine_name',
)


class RandomizedRegionSerializer(RegionSerializer):
    '''
    Adds slug relation to the randomized Region for pools using the
    machine_name field as the value and also
    excludes the primary key since it is meaningless for clients.
    '''
    class Meta(object):
        model = fp_models.RandomizedRegion
        exclude = ('id', )


class RandomizedRegionHyperlinkedSerializer(RandomizedRegionSerializer):
    '''
    Adds URL relation to the randomized Region for pools and also
    excludes the primary key since it is meaningless for clients.
    '''
    pool = serializers.HyperlinkedRelatedField(view_name='pool-detail',
                                               read_only=True,
                                               lookup_field='machine_name')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from random_flavorpack.api.urls import urlpatterns
from quartet_capture.urls import urlpatterns as capture_patterns
from serialbox.api.urls import urlpatterns as sb_patterns

urlpatterns = urlpatterns + capture_patterns + sb_patterns



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Copyright 2018 SerialLab Corp.  All rights reserved.
from django.utils.translation import gettext as _
from django.core.management.base import BaseCommand
from django.contrib.auth.models import Group, Permission, ContentType
from django.db.utils import IntegrityError
from serialbox import models

class Command(BaseCommand):
    help = _(
        'Loads default groups into the database.'
    )

    def handle(self, *args, **options):
        ct = ContentType.objects.get_for_model(models.Pool)
        pool_allocate, created = Permission.objects.get_or_create(
            codename='allocate_numbers',
            content_type=ct
        )
        if created:
            pool_allocate.name='Can allocate numbers'
            pool_allocate.save()

        group = Group.objects.get_or_create(
            name='Pool API Access'
        )[0]
        self._add_permission(group,
           pool_allocate
        )
        self._add_permission(group,
            Permission.objects.get(codename='add_pool')
        )
        self._add_permission(group,
            Permission.objects.get(codename='change_pool')
        )
        self._add_permission(group,
            Permission.objects.get(codename='delete_pool')
        )
        self._add_permission(group,
            Permission.objects.get(codename='add_randomizedregion')
        )
        self._add_permission(group,
            Permission.objects.get(codename='change_randomizedregion')
        )
        self._add_permission(group,
            Permission.objects.get(codename='delete_randomizedregion')
        )
        for perm in group.permissions.all():
            print(perm.name)

    def _add_permission(self, group: Group, permission: Permission):
        try:
            group.permissions.add(permission)
        except IntegrityError:
            print('Permission %s already exists' % permission.name)



'''
    Copyright 2018 SerialLab, CORP

    This file is part of RandomFlavorpack.

    RandomFlavorpack is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RandomFlavorpack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with RandomFlavorpack.  If not, see <http://www.gnu.org/licenses/>.
'''

from django.conf.urls import url

from random_flavorpack.api import viewsets
from random_flavorpack.api.routers import router

urlpatterns = router.urls

# urlpatterns = [
#     url(r'^randomized-regions/$',
#         viewsets.randomized_region_list,
#         name='randomized-region-list'),
#     url(r'^randomized-region-create/$',
#         viewsets.randomized_region_create,
#         name='randomized-region-create'),
#     url(r'^randomized-region-detail/(?P<machine_name>[0-9a-zA-Z]{1,100})/$',
#         viewsets.randomized_region_detail,
#         name='randomized-region-detail'),
#     url(r'^randomized-region-modify/(?P<machine_name>[0-9a-zA-Z]{1,100})/$',
#         viewsets.randomized_region_modify,
#         name='randomized-region-modify'),
#     url(r'^randomized-region-form/(?P<machine_name>[0-9a-zA-Z]{1,100})/$',
#         viewsets.randomized_region_form,
#         name='randomized-region-form'),
# ]
